using System;
using System.Runtime.InteropServices;
using System.Text;

namespace epmux {
	public enum Result {
	    ok = 0,
	    error = -1,
	    not_implemented = -2,
	    value_error = -3,
	    no_device = -4
	};
	public static class UrpcBuilderVersion {
	    public const int major = 0;
	    public const int minor = 10;
	    public const int bugfix = 12;
	    public const String suffix = "";
	    public const String version = "0.10.12";
	};
	[Flags] public enum Flags : uint {
		MAX_CHAIN_LENGTH = 16,
		MODULE_TYPE_A = 1,
		MODULE_TYPE_AB = 2,
		NO_MODULE = 0,
	}
	[StructLayout(LayoutKind.Sequential, CharSet=CharSet.Ansi)]
	public struct get_identity_information_t {
		[MarshalAs(UnmanagedType.ByValArray, SizeConst = 16)]
		public byte[] Manufacturer;
		[MarshalAs(UnmanagedType.ByValArray, SizeConst = 16)]
		public byte[] ProductName;
		[MarshalAs(UnmanagedType.ByValArray, SizeConst = 16)]
		public byte[] ControllerName;
		public byte HardwareMajor;
		public byte HardwareMinor;
		public ushort HardwareBugfix;
		public byte BootloaderMajor;
		public byte BootloaderMinor;
		public ushort BootloaderBugfix;
		public byte FirmwareMajor;
		public byte FirmwareMinor;
		public ushort FirmwareBugfix;
		public uint SerialNumber;
		[MarshalAs(UnmanagedType.ByValArray, SizeConst = 8)]
		public byte[] Reserved;
	};
	[StructLayout(LayoutKind.Sequential, CharSet=CharSet.Ansi)]
	public struct debug_read_t {
		[MarshalAs(UnmanagedType.ByValArray, SizeConst = 128)]
		public byte[] DebugData;
		[MarshalAs(UnmanagedType.ByValArray, SizeConst = 8)]
		public byte[] Reserved;
	};
	[StructLayout(LayoutKind.Sequential, CharSet=CharSet.Ansi)]
	public struct debug_write_t {
		[MarshalAs(UnmanagedType.ByValArray, SizeConst = 128)]
		public byte[] DebugData;
		[MarshalAs(UnmanagedType.ByValArray, SizeConst = 8)]
		public byte[] Reserved;
	};
	[StructLayout(LayoutKind.Sequential, CharSet=CharSet.Ansi)]
	public struct get_chain_structure_t {
		public byte ChainLength;
		[MarshalAs(UnmanagedType.ByValArray, SizeConst = 16)]
		public byte[] ChainStructure;
		[MarshalAs(UnmanagedType.ByValArray, SizeConst = 16)]
		public byte[] Reserved;
	};
	[StructLayout(LayoutKind.Sequential, CharSet=CharSet.Ansi)]
	public struct channel_for_line_a_t {
		public byte ModuleNumber;
		public byte ChannelNumber;
		[MarshalAs(UnmanagedType.ByValArray, SizeConst = 14)]
		public byte[] Reserved;
	};
	[StructLayout(LayoutKind.Sequential, CharSet=CharSet.Ansi)]
	public struct channel_for_line_b_t {
		public byte ModuleNumber;
		public byte ChannelNumber;
		[MarshalAs(UnmanagedType.ByValArray, SizeConst = 14)]
		public byte[] Reserved;
	};
	public partial class API {
		[DllImport("epmux", CharSet = CharSet.Ansi, CallingConvention=CallingConvention.Cdecl)]
		public static extern int epmux_open_device([MarshalAs(UnmanagedType.LPStr)] String name);
		[DllImport("epmux", CharSet = CharSet.Ansi, CallingConvention=CallingConvention.Cdecl)]
		public static extern int epmux_close_device([MarshalAs(UnmanagedType.LPStr)] String libver);
		[DllImport("epmux", CallingConvention=CallingConvention.Cdecl)]
		public static extern Result epmux_close_device(ref int id);
		[DllImport("epmux", CallingConvention=CallingConvention.Cdecl)]
		public static extern Result epmux_save_settings(int id);
		[DllImport("epmux", CallingConvention=CallingConvention.Cdecl)]
		public static extern Result epmux_read_settings(int id);
		[DllImport("epmux", CallingConvention=CallingConvention.Cdecl)]
		public static extern Result epmux_get_identity_information(int id, out get_identity_information_t output);
		[DllImport("epmux", CallingConvention=CallingConvention.Cdecl)]
		public static extern Result epmux_reboot_to_bootloader(int id);
		[DllImport("epmux", CallingConvention=CallingConvention.Cdecl)]
		public static extern Result epmux_debug_read(int id, out debug_read_t output);
		[DllImport("epmux", CallingConvention=CallingConvention.Cdecl)]
		public static extern Result epmux_debug_write(int id, ref debug_write_t input);
		[DllImport("epmux", CallingConvention=CallingConvention.Cdecl)]
		public static extern Result epmux_reset(int id);
		[DllImport("epmux", CallingConvention=CallingConvention.Cdecl)]
		public static extern Result epmux_get_channel_for_line_a(int id, out channel_for_line_a_t output);
		[DllImport("epmux", CallingConvention=CallingConvention.Cdecl)]
		public static extern Result epmux_set_channel_for_line_a(int id, ref channel_for_line_a_t input);
		[DllImport("epmux", CallingConvention=CallingConvention.Cdecl)]
		public static extern Result epmux_get_chain_structure(int id, out get_chain_structure_t output);
		[DllImport("epmux", CallingConvention=CallingConvention.Cdecl)]
		public static extern Result epmux_all_channels_off(int id);
		[DllImport("epmux", CallingConvention=CallingConvention.Cdecl)]
		public static extern Result epmux_get_channel_for_line_b(int id, out channel_for_line_b_t output);
		[DllImport("epmux", CallingConvention=CallingConvention.Cdecl)]
		public static extern Result epmux_set_channel_for_line_b(int id, ref channel_for_line_b_t input);
	};
};